==> openejb-core-4.5.1-sources.jar: org/apache/openejb/assembler/classic/LazyValidator.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.openejb.assembler.classic;

import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.util.concurrent.locks.ReentrantLock;

public class LazyValidator implements InvocationHandler {

    private final ReentrantLock lock = new ReentrantLock();
    private final ValidatorFactory factory;
    private Validator validator = null;

    public LazyValidator(final ValidatorFactory factory) {
        this.factory = factory;
    }

    @Override
    public Object invoke(final Object proxy, final Method method, final Object[] args) throws Throwable {

        if (validator == null) {

            final ReentrantLock l = lock;
            l.lock();

            try {
                if (validator == null) {
                    validator = factory.usingContext().getValidator();
                }
            } finally {
                l.unlock();
            }
        }

        return method.invoke(validator, args);
    }
}

==> openejb-core-4.7.1-sources.jar: org/apache/openejb/assembler/classic/LazyValidator.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.openejb.assembler.classic;

import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.util.concurrent.locks.ReentrantLock;

public class LazyValidator implements InvocationHandler {

    private final ReentrantLock lock = new ReentrantLock();
    private final ValidatorFactory factory;
    private Validator validator;

    public LazyValidator(final ValidatorFactory factory) {
        this.factory = factory;
    }

    @Override
    public Object invoke(final Object proxy, final Method method, final Object[] args) throws Throwable {

        if (validator == null) {

            final ReentrantLock l = lock;
            l.lock();

            try {
                if (validator == null) {
                    validator = factory.usingContext().getValidator();
                }
            } finally {
                l.unlock();
            }
        }

        return method.invoke(validator, args);
    }
}
